'''
tests for std_name_table module
'''

import os

from nose import tools as nt

from CFchecker.std_name_table import StandardNameTable
from CFchecker.errors import StandardNameTableNotFoundError, StandardNameTableCorruptError

_tempdir = '/tmp/'
_moduledir = os.path.dirname(__file__)

class StandardNameTable_test:
    test_name_table = os.path.join(_moduledir, 'test_std_name_table.xml')
    nonexistent_name_table = os.path.join(_tempdir, 'tests_nonexistent_name_table.xml')
    corrupt_name_table = os.path.join(_moduledir, 'test_std_name_table_corr.xml')

    @classmethod
    def setup_class(cls):
        if os.path.exists(cls.nonexistent_name_table): os.remove(cls.nonexistent_name_table)
        if not os.path.exists(cls.test_name_table):
            raise Exception('name table test input file %s not found' % cls.test_name_table)
    def test_init_ok(self):
        '''NameTable: init with valid table file'''
        name_table = StandardNameTable(self.test_name_table)
        nt.assert_equal(set(name_table.names), set(['air_potential_temperature', 'atmosphere_hybrid_sigma_pressure_coordinate',
                                                    'wind_speed_of_gust', 'longitude', 'surface_temperature', 'time','area_type',
                                                    'latitude', 'age_of_stratospheric_air', 'atmosphere_sulfate_content',
                                                    'atmosphere_ln_pressure_coordinate']))
        nt.assert_equal(set(name_table.aliases), set(['surface_temperature_where_land', 'atmosphere_so4_content']))
    def test_init_nofile(self):
        '''NameTable: init with non-existing file'''
        nt.assert_raises(StandardNameTableNotFoundError, StandardNameTable, self.nonexistent_name_table)
    def test_get_version(self):
        '''NameTable: get_version() reports correct version'''
        name_table = StandardNameTable(self.test_name_table)
        nt.assert_equals(name_table.get_version(), '12')
    def test_get_last_modified(self):
        '''NameTable: get_last_modified() reports correct date'''
        name_table = StandardNameTable(self.test_name_table)
        nt.assert_equals(name_table.get_last_modified(), '2009-07-06T12:05:02Z')
    def test_exists_ok(self):
        '''NameTable: entries are found by exists()'''
        name_table = StandardNameTable(self.test_name_table)
        names = ['age_of_stratospheric_air', 'air_potential_temperature', 'atmosphere_sulfate_content', 'atmosphere_so4_content']
        for name in names:
            nt.assert_true(name_table.exists(name))
    def test_exists_bad(self):
        '''NameTable: non-existing names are not found by exists()'''
        name_table = StandardNameTable(self.test_name_table)
        names = ['foo', 'bar', 'baz']
        for name in names:
            nt.assert_false(name_table.exists(name))
    def test_is_alias_ok(self):
        '''NameTable: is_alias() works for aliases'''
        name_table = StandardNameTable(self.test_name_table)
        aliases = ['atmosphere_so4_content']
        for alias in aliases:
            nt.assert_true(name_table.is_alias(alias))
    def test_is_alias_bad(self):
        '''NameTable: is_alias works for non-alias names'''
        name_table = StandardNameTable(self.test_name_table)
        names = ['age_of_stratospheric_air', 'air_potential_temperature', 'atmosphere_sulfate_content']
        for name in names:
            nt.assert_false(name_table.is_alias(name), 'real name reported as alias')
        names = ['foo', 'bar', 'baz']
        for name in names:
            nt.assert_false(name_table.is_alias(name), 'nonexistent name reported as alias')
    def test_lookup_alias_ok(self):
        '''NameTable: lookup_alias() works for aliases'''
        name_table = StandardNameTable(self.test_name_table)
        aliases = ['atmosphere_so4_content']
        names = ['atmosphere_sulfate_content']
        for alias, name in zip(aliases,names):
            nt.assert_equal(name_table.lookup_alias(alias), name)
    def test_lookup_alias_bad(self):
        '''NameTable: lookup_alias() works for non-alias names'''
        name_table = StandardNameTable(self.test_name_table)
        aliases = ['bar','spam']
        for alias in aliases:
            nt.assert_raises(StandardNameTableNotFoundError, name_table.lookup_alias, alias)
        names = ['age_of_stratospheric_air', 'air_potential_temperature', 'atmosphere_sulfate_content']
        for name in names:
            nt.assert_raises(StandardNameTableNotFoundError, name_table.lookup_alias, name)
    def test_lookup_alias_corrupt(self):
        '''NameTable: lookup_alias() reports corrupted name table'''
        name_table = StandardNameTable(self.corrupt_name_table)
        aliases = ['alias_with_two_real_names', 'alias_without_real_name']
        for alias in aliases:
            nt.assert_raises(StandardNameTableCorruptError, name_table.lookup_alias, alias)
    def test_get_units_ok(self):
        '''NameTable: get_units() works for valid names'''
        name_table = StandardNameTable(self.test_name_table)
        names = ['age_of_stratospheric_air', 'air_potential_temperature', 'atmosphere_sulfate_content', 'atmosphere_so4_content']
        units = ['s', 'K', 'kg m-2', 'kg m-2']
        for name,unit in zip(names, units):
            nt.assert_equal(name_table.get_units(name), unit)
    def test_get_units_bad(self):
        '''NameTable: get_units() works for invalid names'''
        name_table = StandardNameTable(self.test_name_table)
        names = ['foo', 'bar', 'baz']
        for name in names:
            nt.assert_raises(StandardNameTableNotFoundError, name_table.get_units, name)
    def test_get_units_corrupt(self):
        '''NameTable: get_units() reports corrupt name table'''
        name_table = StandardNameTable(self.corrupt_name_table)
        names = ('air_potential_temperature_no_units', 'air_potential_temperature_double_units')
        for name in names:
            nt.assert_raises(StandardNameTableCorruptError, name_table.get_units, name)
